/**
 * ==============================================
 *  DQL : Descartes Query Language
 * ==============================================
 *
 * (c) Copyright 2014-2016, by Juergen Walter, Fabian Gorsler, Fabian Brosig and other contributors.
 *
 * Project Info:   http://descartes.tools/dql
 *
 * All rights reserved. This software is made available under the terms of the
 * Eclipse Public License (EPL) v1.0 as published by the Eclipse Foundation
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * This software is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
 * or FITNESS FOR A PARTICULAR PURPOSE. See the Eclipse Public License (EPL)
 * for more details.
 *
 * You should have received a copy of the Eclipse Public License (EPL)
 * along with this software; if not visit http://www.eclipse.org or write to
 * Eclipse Foundation, Inc., 308 SW First Avenue, Suite 110, Portland, 97204 USA
 * Email: license (at) eclipse.org
 *
 * [Java is a trademark or registered trademark of Sun Microsystems, Inc.
 * in the United States and other countries.]
 */
package tools.descartes.dql.core.engine.util;

import java.awt.BorderLayout;
import java.io.BufferedWriter;
import java.io.File;
import java.io.FileWriter;
import java.io.IOException;
import java.util.HashMap;

import javax.swing.JFrame;
import javax.swing.JPanel;
import javax.swing.JTextArea;

import org.apache.log4j.Logger;
import org.eclipse.emf.common.util.EList;

import tools.descartes.dql.lang.descartesQL.Sla;
import tools.descartes.dql.lang.descartesQL.Slo;

public class SLAReporter {
	protected Logger log = DQLLogger.getLogger(this.getClass().getName());
	private static boolean isQuantitativeAnalysis = true;
	private HashMap<Sla, Double> slaPenaltiesMap;
	private HashMap<Slo, Double> sloPenaltiesMap;
	private HashMap<Slo, Boolean> sloComplianceMap;
	private HashMap<Sla, Boolean> slaComplianceMap;
	EList<Sla> slas;
	String fileName;

	public static void printResults(HashMap<Sla, Double> slaPenaltiesMap, HashMap<Slo, Double> sloPenaltiesMap,
			HashMap<Slo, Boolean> sloComplianceMap, HashMap<Sla, Boolean> slaComplianceMap, EList<Sla> slas,
			String fileName) {
		SLAReporter reporter = new SLAReporter(slaPenaltiesMap, sloPenaltiesMap, sloComplianceMap, slaComplianceMap,
				slas, fileName);
		reporter.logResults();
		reporter.createReportWindow();
		reporter.writeReport();
	}

	public SLAReporter(HashMap<Sla, Double> slaPenaltiesMap, HashMap<Slo, Double> sloPenaltiesMap,
			HashMap<Slo, Boolean> sloComplianceMap, HashMap<Sla, Boolean> slaComplianceMap, EList<Sla> slas,
			String fileName) {
		this.slaComplianceMap = slaComplianceMap;
		this.sloComplianceMap = sloComplianceMap;
		this.slaPenaltiesMap = slaPenaltiesMap;
		this.sloPenaltiesMap = sloPenaltiesMap;
		this.slas = slas;
		this.fileName = fileName.substring(0, fileName.lastIndexOf(File.separatorChar)) + File.separator
				+ "report.txt";
	}

	private void logResults() {
		log.info("------------------------------------");
		log.info("------------------------------------");
		log.info("REPORT");
		log.info("Number of SLAs: " + slas.size());
		log.info("Number of analyzed SLAs: " + slaComplianceMap.keySet().size());

		for (Sla sla : slaComplianceMap.keySet()) {
			if (!slaComplianceMap.get(sla)) {
				if (!isQuantitativeAnalysis) {
					log.info("\t" + "Execution comlies NOT to " + sla.getName());
				} else {
					log.info("\t" + "Execution comlies NOT to " + sla.getName() + ", penalties "
							+ slaPenaltiesMap.get(sla));
				}
			} else {
				log.info("\t" + "Execution comlies to " + sla.getName());
			}
		}

		log.info("------");
		for (Slo slo : sloComplianceMap.keySet()) {
			if (!sloComplianceMap.get(slo)) {
				if (!isQuantitativeAnalysis) {
					log.info(slo.getName() + " compliance: " + sloComplianceMap.get(slo));
				} else {
					String currency = "";
					try {
						currency = slo.getPen().getCurrency().getName();
					} catch (NullPointerException e) {
						// TODO: handle exception
					}
					log.info(slo.getName() + " compliance: " + sloComplianceMap.get(slo) + ", penalties "
							+ sloPenaltiesMap.get(slo) + currency);
				}
			} else {
				log.info("Execution comlies to " + slo.getName());
			}
		}
		log.info("------------------------------------");
		log.info("------------------------------------");

	}

	private void createReportWindow() {
		JFrame frame = new JFrame();
		JPanel panel = new JPanel();
		frame.add(panel);

		frame.setVisible(true);
		frame.setTitle("SLA report\n");

		JTextArea label = new JTextArea();
		panel.add(label, BorderLayout.CENTER);
		label.setText(getString());

		JTextArea label2 = new JTextArea();
		panel.add(label2, BorderLayout.CENTER);
		label2.setText("Save copy of report to " + fileName);
		frame.pack();
	}

	private void writeReport() {
		writeToFile(fileName, getString());

	}

	private void writeToFile(String file, String content) {
		BufferedWriter writer = null;
		try {
			writer = new BufferedWriter(new FileWriter(file));
			writer.write(content);
			System.out.println("-------------------");
			System.out.println(content);
			System.out.println("-------------------");
			System.out.println("Done");
		} catch (IOException e) {
			log.error(e);
		} finally {
			try {
				if (writer != null)
					writer.close();
			} catch (IOException e) {
				log.error(e);
			}
		}
	}

	private String getString() {
		StringBuffer sb = new StringBuffer();
		sb.append("------------------------------------\n");
		sb.append("------------------------------------\n");
		sb.append("REPORT\n");
		sb.append("Number of SLAs: " + slas.size() + "\n");
		sb.append("Number of analyzed SLAs: " + slaComplianceMap.keySet().size() + "\n");

		for (Sla sla : slaComplianceMap.keySet()) {
			if (!slaComplianceMap.get(sla)) {
				if (!isQuantitativeAnalysis) {
					sb.append("\t" + "Execution comlies NOT to " + sla.getName() + "\n");
				} else {
					sb.append("\t" + "Execution comlies NOT to " + sla.getName() + ", penalties "
							+ slaPenaltiesMap.get(sla) + "\n");
				}
			} else {
				sb.append("\t" + "Execution comlies to " + sla.getName() + "\n");
			}
		}

		sb.append("------\n");
		for (Slo slo : sloComplianceMap.keySet()) {
			if (!sloComplianceMap.get(slo)) {
				if (!isQuantitativeAnalysis) {
					sb.append(slo.getName() + " compliance: " + sloComplianceMap.get(slo) + "\n");
				} else {
					String currency = "";
					try {
						currency = slo.getPen().getCurrency().getName();
					} catch (NullPointerException e) {
						// TODO: handle exception
					}
					sb.append(slo.getName() + " compliance: " + sloComplianceMap.get(slo) + ", penalties "
							+ sloPenaltiesMap.get(slo) + currency + "\n");
				}
			} else {
				sb.append("Execution comlies to " + slo.getName() + "\n");
			}
		}
		sb.append("------------------------------------\n");
		sb.append("------------------------------------\n");
		return sb.toString();
	}

}
